<?php
declare(strict_types=1);

namespace App\Form;


use App\Exception\InvalidColorException;
use App\Model\ColorVo;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\DataMapperInterface;
use Symfony\Component\Form\DataTransformerInterface;
use Symfony\Component\Form\Exception;
use Symfony\Component\Form\Exception\TransformationFailedException;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Validator\Context\ExecutionContextInterface;

class ColorType extends AbstractType implements DataTransformerInterface
{
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder->addModelTransformer($this);
    }

    /**
     * {@inheritdoc}
     */
    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setDefaults([
            'compound' => false,
        ]);
    }

    public function transform($value)
    {
        if ($value === null) {
            return null;
        }
        return $value->getHexaColor();
    }

    public function reverseTransform($value)
    {
        if ($value === null) {
            return null;
        }

        return new ColorVo($value);
    }

    public function mapDataToForms($viewData, iterable $forms)
    {
        // Will obviously not work but should
        $forms->setData($viewData->getHexaColor());
    }

    public function mapFormsToData(iterable $forms, &$viewData)
    {
        $viewData = new ColorVo($forms->getData());
    }

    public static function assertColorIsValid(string $color): void
    {
        if (preg_match('#^\#[0-9]{6}$#', $color) === 1) {
            return;
        }

        throw new InvalidColorException('Invalid color');
    }

    public static function validateColor($color, ExecutionContextInterface $context, $payload)
    {
        try {
            self::assertColorIsValid($color);
        } catch (InvalidColorException $e) {
            $context->buildViolation('Impossible color bruuuu')
                ->addViolation()
            ;
        }
    }
}
